require "util"
require "config"

script.on_init(function()
	global.growing_saplings_record = {}
	global.seed_fall_record = {}
end)

script.on_configuration_changed(function(ConfigurationChangedData)
	
end)

script.on_event(defines.events.on_built_entity, function(event)
	if event.created_entity.name == "planted-sapling" then
		add_sapling_to_record(event.created_entity.position, event.tick, MAX_PROPERGATIONS)
	end
end)

script.on_event(defines.events.on_tick, function(event)
	local tick = event.tick
	if tick % TICKS_BETWEEN_UPDATES == 0 then
		update_saplings(tick)
	end
end)

script.on_event(defines.events.on_entity_died, function(event)
	if event.entity.name == "planted-sapling" then
		remove_sapling_from_record(event.entity.position)
	else
		if event.entity.type == "tree" then
			distribute_falling_seeds(event.entity.position, MAX_PROPERGATIONS)
		end
	end
end)

script.on_event(defines.events.on_preplayer_mined_item, function(event)
	if event.entity.name == "planted-sapling" then
		remove_sapling_from_record(event.position)
	end
end)



function update_saplings(tick)
	local pos
	
	if #global.growing_saplings_record > 0 then
		if tick > global.growing_saplings_record[1].tick then 
			grow_tree(global.growing_saplings_record[1].position)
		end
	end

	if #global.seed_fall_record > 0 then
		if tick > global.seed_fall_record[1].tick then
			pos = global.seed_fall_record[1].position
		
			if game.surfaces.nauvis.can_place_entity({name = "planted-sapling", position = pos}) then
				game.surfaces.nauvis.create_entity({name = "planted-sapling", position = pos})
				add_sapling_to_record(pos, tick, global.seed_fall_record[1].remaining)
				table.remove(global.seed_fall_record, 1)
			else
				table.remove(global.seed_fall_record, 1)
			end
		end
	end
end


function add_sapling_to_record(pos, tick, cycles)
	local tile_name = game.surfaces.nauvis.get_tile(pos.x, pos.y).name
	local valid_tile = true
	
	for index, dislike in pairs(TREES_DONT_GROW_ON) do
		if string.find(tile_name, dislike) ~= nil then
			valid_tile = false
		end
	end
	
	if valid_tile == true then 
		table.insert(global.growing_saplings_record, 
													{	
														position = {x = pos.x, y = pos.y}, 
														tick = tick + TICKS_TO_GROW,
														remaining = cycles
													})
	else
		game.surfaces.nauvis.find_entity("planted-sapling", {pos.x, pos.y}).destroy()
	end
end


function distribute_falling_seeds(pos, cycles)
	local tempx = 0
	local tempy = 0
	local num_seeds = math.floor(math.random(MAX_SEEDS_FALL +1)) -1
	
	if num_seeds > 0 then
		for index = 1, num_seeds do
			tempx = math.floor(pos.x + (math.random(MAX_SEED_FALL_DISTANCE) * random_modifier())) + 0.5
			tempy = math.floor(pos.y + (math.random(MAX_SEED_FALL_DISTANCE) * random_modifier())) + 0.5
			table.insert(global.seed_fall_record, 
						{
							position = {x = tempx, y = tempy} ,
							tick = game.tick + TICKS_TO_GERMINATE,
							remaining = cycles
						})
		end
	end
end


function remove_sapling_from_record(pos)
	if global.growing_saplings_record ~= nil and pos ~= nil then
		if pos.x ~= nil and pos.y ~= nil then
			if #global.growing_saplings_record > 0 then
				for index, sapling in pairs(global.growing_saplings_record) do	
					if sapling.position.x == pos.x and sapling.position.y == pos.y then
						table.remove(global.growing_saplings_record, index)
						return
					end
				end
			end
		end
	end
end


function grow_tree(pos)
	local cycles
	sapling = game.surfaces.nauvis.find_entity("planted-sapling", {pos.x, pos.y})
	
	if sapling ~= nil then
		if global.growing_saplings_record[1].remaining ~= nil then
			cycles = global.growing_saplings_record[1].remaining - 1
		else
			cycles = 0
		end

		if cycles > 0 and math.floor(math.random(100)) < REPROPERGATION_CHANCE then
			distribute_falling_seeds(pos, cycles)
		end
		
		sapling.destroy()
		tree = TREE_NAMES[math.random(#TREE_NAMES)]
		
		if game.surfaces.nauvis.can_place_entity({name = tree, position = pos}) then
			game.surfaces.nauvis.create_entity({name = tree, position = pos})
			table.remove(global.growing_saplings_record, 1)
		else
			table.remove(global.growing_saplings_record, 1)
		end
	else
		table.remove(global.growing_saplings_record, 1)
	end
end

function random_modifier()
	local temp = math.random(100)%2
	
	if temp == 0 then
		temp = -1
	end
	
	return temp
end

